-------------------------------------------------------------------------------
-- Locals                                                                    --
-------------------------------------------------------------------------------

local SEM = AceLibrary("SpecialEvents-Mount-2.0")
local L = AceLibrary("AceLocale-2.2"):new("ClosetGnomeMount")
local BZ = AceLibrary("Babble-Zone-2.2")

local CG = ClosetGnome

local inBattleground = nil
local lastSetBeforeMounting = nil
local mounted = nil

-------------------------------------------------------------------------------
-- Localization                                                              --
-------------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Mount"] = true,
	["Dismount"] = true,
	["Adds a saddle to your mount that can carry your ClosetGnome."] = true,

	["Battleground"] = true,
	["Toggle whether set switching when mounting is enabled in battlegrounds or not."] = true,
	["What set to equip when you dismount. If no set is specified, the set you equipped before you mounted will be equipped automatically on dismount."] = true,
	["What set to equip when you mount."] = true,
	["<set>"] = true,

	["Could not find a set named %q. Please add one."] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Mount"] = "骑乘换装",
	["Could not find a set named %q. Please add one."] = "无法找到名为 qs 的设置. 请添加一个.",
} end)

L:RegisterTranslations("koKR", function() return {
	["Mount"] = "타기",
	["Dismount"] = "내리기",

	["Battleground"] = "전장",
	["Toggle whether set switching when mounting is enabled in battlegrounds or not."] = "전장에서 세트 사용 여부를 토글합니다.",
	--["What set to equip when you dismount."] = "당신이 내릴때 준비될 세트",
	["What set to equip when you mount."] = "당신이 탈때 준비될 세트",
	["<set>"] = "<세트>",

	["Could not find a set named %q. Please add one."] = "세트 이름 %q|1을;를 찾을수 없습니다. 하나를 추가하세요.",
} end)

L:RegisterTranslations("esES", function() return {
	["Mount"] = "Montar",
	["Dismount"] = "Desmontar",
	["Adds a saddle to your mount that can carry your ClosetGnome."] = "A\195\177ade una silla de montar en la que puedes llevar a tu ClosetGnome.",

	["Battleground"] = "Campo de Batalla",
	["Toggle whether set switching when mounting is enabled in battlegrounds or not."] = "Determina si se activa este accesorio en campos de batalla",
	["What set to equip when you dismount. If no set is specified, the set you equipped before you mounted will be equipped automatically on dismount."] = "Qu\195\169 conjunto equipar cuando desmontas. Si no se especifica un conjunto se equipar\195\161 autom\195\161ticamente al desmontar el conjunto que ten\195\173as puesto antes de montar",
	["What set to equip when you mount."] = "Qu\195\169 conjunto equipar cuando montas.",
	["<set>"] = "<conjunto>",

	["Could not find a set named %q. Please add one."] = "No se ha podido encontrar el conjunto llamado %q. Por favor, a\195\177ade uno.",
} end)

-------------------------------------------------------------------------------
-- Addon declaration                                                         --
-------------------------------------------------------------------------------

ClosetGnomeMount = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "AceHook-2.1")

local options = {
	type = "group",
	name = L["Mount"],
	desc = L["Adds a saddle to your mount that can carry your ClosetGnome."],
	order = 106,
	args = {
		[L["Dismount"]] = {
			type = "text",
			name = L["Dismount"],
			desc = L["What set to equip when you dismount. If no set is specified, the set you equipped before you mounted will be equipped automatically on dismount."],
			get = function() return ClosetGnomeMount.db.char.dismount end,
			set = function(v)
				if v:trim() == "" then v = nil end
				ClosetGnomeMount.db.char.dismount = v
			end,
			validate = function(input) return type(input) == "string" or input == nil end,
			usage = L["<set>"],
		},
		[L["Mount"]] = {
			type = "text",
			name = L["Mount"],
			desc = L["What set to equip when you mount."],
			get = function() return ClosetGnomeMount.db.char.mount end,
			set = function(v)
				if v:trim() == "" then v = nil end
				ClosetGnomeMount.db.char.mount = v
			end,
			validate = function(input) return type(input) == "string" or input == nil end,
			usage = L["<set>"],
		},
		[L["Battleground"]] = {
			type = "toggle",
			name = L["Battleground"],
			desc = L["Toggle whether set switching when mounting is enabled in battlegrounds or not."],
			get = function() return ClosetGnomeMount.db.char.enabledInBG end,
			set = function(v)
				ClosetGnomeMount:ToggleBattleground(v)
			end,
		},
	},
}

-------------------------------------------------------------------------------
-- Initialization                                                            --
-------------------------------------------------------------------------------

function ClosetGnomeMount:OnInitialize()
	if not CG then return end

	self:RegisterDB("ClosetGnomeMountDB", "ClosetGnomeMountPCDB", "char")
	self:RegisterDefaults("char", {
		dismount = nil,
		mount = nil,
		enabledInBG = true,
		mountedOnLogout = nil,
	})

	options.args.about = nil
	CG.OnMenuRequest.args.mount = options
end

function ClosetGnomeMount:OnEnable()
	if not CG then return end

	self:RegisterEvent("SpecialEvents_Mounted")
	self:RegisterEvent("SpecialEvents_Dismounted")
	self:RegisterEvent("ClosetGnome_WearSet")

	self:RegisterEvent("PLAYER_LOGOUT")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")

	self:Hook(CG, "HideSetFromUI")

	self:ZONE_CHANGED_NEW_AREA()

	if not CG:HasSet(self:GetMountSet()) then
		self:Print(L["Could not find a set named %q. Please add one."]:format(self:GetMountSet()))
	else
		-- Force a menu update to hide the mount set properly.
		CG:UpdateSetMenu()
	end

	if self.db.profile.mountedOnLogout then
		lastSetBeforeMounting = self.db.profile.mountedOnLogout
		self.db.profile.mountedOnLogout = nil
	end
end

-------------------------------------------------------------------------------
-- Util                                                                      --
-------------------------------------------------------------------------------

local battlegrounds = {
	[BZ["Alterac Valley"]] = true,
	[BZ["Arathi Basin"]] = true,
	[BZ["Warsong Gulch"]] = true,
	[BZ["Nagrand Arena"]] = true,
	[BZ["Eye of the Storm"]] = true,
	[BZ["Blade's Edge Arena"]] = true,
	[BZ["Ruins of Lordaeron"]] = true,
}

local function InBattleground()
	return battlegrounds[GetRealZoneText()] or battlegrounds[GetZoneText()] or nil
end

function ClosetGnomeMount:GetMountSet()
	local x = self.db.char.mount
	if type(x) == "string" and x:trim() == "" then x = nil end
	return x or L["Mount"]
end

function ClosetGnomeMount:GetDismountSet()
	local x = self.db.char.dismount
	if type(x) == "string" and x:trim() == "" then x = nil end
	return x or L["Dismount"]
end

function ClosetGnomeMount:ToggleBattleground(enabledInBG)
	self.db.char.enabledInBG = enabledInBG and true or false

	if not enabledInBG then
		if self:IsEventRegistered("ZONE_CHANGED_NEW_AREA") then
			self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
			inBattleground = nil
		end
	else
		if not self:IsEventRegistered("ZONE_CHANGED_NEW_AREA") then
			self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		end
		self:ZONE_CHANGED_NEW_AREA()
	end
end

local function Wear(set)
	local quips = CG.db.profile.quips
	CG.db.profile.quips = nil
	CG:WearSet(set)
	CG.db.profile.quips = quips
end

-------------------------------------------------------------------------------
-- Events                                                                    --
-------------------------------------------------------------------------------

function ClosetGnomeMount:ZONE_CHANGED_NEW_AREA()
	inBattleground = InBattleground()
end

function ClosetGnomeMount:PLAYER_LOGOUT()
	if mounted then
		self.db.profile.mountedOnLogout = lastSetBeforeMounting
	end
end

function ClosetGnomeMount:HideSetFromUI(_, set)
	if set and set == L["Mount"] or set == L["Dismount"] then
		return true
	end
	return self.hooks[CG]["HideSetFromUI"](_, set)
end

function ClosetGnomeMount:ClosetGnome_WearSet(name)
	if name == self:GetMountSet() or name == self:GetDismountSet() then return end
	lastSetBeforeMounting = name
end

function ClosetGnomeMount:SpecialEvents_Mounted()
	if IsControlKeyDown() or (inBattleground and not self.db.profile.enabledInBG) then return end

	local set = self:GetMountSet()
	if not CG:HasSet(set) then
		self:Print(L["Could not find a set named %q. Please add one."]:format(set))
		return
	end
	Wear(set)
	mounted = true
end

function ClosetGnomeMount:SpecialEvents_Dismounted()
	if IsControlKeyDown() or (inBattleground and not self.db.profile.enabledInBG) then return end

	local set = self:GetDismountSet()
	if CG:HasSet(set) then
		Wear(set)
		mounted = nil
	elseif CG:HasSet(lastSetBeforeMounting) then
		Wear(lastSetBeforeMounting)
		mounted = nil
	end
end

